
const COMMAEncode = function(row) {
  var rowstr = '';
  var i;
  for(i=0; i < row.length; ++i) {
    rowstr= rowstr + (row[i] + '').replace(/,/g, '__COMMA__') + ",";
  }
  if (rowstr.length > 0)
    rowstr = rowstr.substr(0, rowstr.length-1);
  return rowstr;
}

const ROWEncode = function(res) {
  var i, ret ='';
  for(i=0; i < res.length; ++i)
    ret += "__ROW__"  + COMMAEncode(res[i]);
  return ret;
}

const runSql = function(q) {
  return new Promise(function(resolve, reject) {
    if (!con) {
      return reject('izy-server-sql is running, however it cannot execure your query because no database is specified. Please make sure you start your session with db=<dbconfig> and make sure the database server for your configuration is accessable.')
    }
    con.query(q, function (err, rows, fields) { 
      if (err)
        return reject(err.toString());

      var ret = [];
      var i, p, row, obj;
      for(i=0; i < rows.length; ++i) {
        row = [];
        obj = rows[i];
        for(p in obj) {
          row.push(obj[p]);
        }
        ret.push(row);
      }
      resolve(ret);
    });
  });
}

var con = null;
const connectToSql = function(cfg) {
  return new Promise(function(resolve, reject) {
    var mysql = require("mysql");
    var cfgs = exports.cfgs;
    if (!cfgs[cfg]) {
      return reject(cfg + ' db config is not supported');
    }
    con = mysql.createConnection(cfgs[cfg]);

    con.connect(function (err) {
      if (err) {
        reject(err);
        return;
      }
      resolve({});
    });
  });
}

var Promise = require('bluebird');

const consumeReqs = function(reqs, cb) {
  return Promise.map(reqs, function(req) {
    console.log('consuming ', req);
    if (req.indexOf('db=') == 0) {
      return connectToSql(req.split('db=')[1]);
    } else {
      return runSql(req);
    }
  }).then(function(res) {
    if (con) {
      con.end(function (err) {
        // Ensures all previously enqueued queries are still
        // before sending a COM_QUIT packet to the MySQL server.
      });
    }
    return res;
  });
}

const handleRequest = (izyreq, cb) => {
  var sep ='__QUERY_SEP__';
  var q = izyreq.query + '';
  q = q.split(sep);
  consumeReqs(q)
    .then(function(res) {
      // Backwards compat -- we only return the last executed statement. use sql vars to do more
      res = res[res.length-1];
      cb({
        success: true,
        // Result needs to be row encoded
        result: ROWEncode(res)
      });
    })
    .catch(function(e) {
      cb({
        success: false,
        reason : e.toString()
      });
    });

  // Yes, we are a catch-all hanlders folks
  return true;
} ;

const canHandleRequest = function(request) {
  return true;
}

exports.cfgs = {
  test: {
    host: 'hostname',
    user: 'username',
    password: 'password',
    database: 'database'
    // debug: true
  }
};

exports.sp = function(key, val) {
  exports[key] = val;
  return exports;
}

exports.handleRequest = handleRequest;
exports.canHandleRequest = canHandleRequest;
